# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.descriptor import (
    Descriptor as google___protobuf___descriptor___Descriptor,
    FieldDescriptor as google___protobuf___descriptor___FieldDescriptor,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from pyatv.mrp.protobuf.TextEditingAttributesMessage_pb2 import (
    TextEditingAttributes as pyatv___mrp___protobuf___TextEditingAttributesMessage_pb2___TextEditingAttributes,
)

from typing import (
    Optional as typing___Optional,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


class KeyboardMessage(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    state = ... # type: int
    encryptedTextCyphertext = ... # type: bytes

    @property
    def attributes(self) -> pyatv___mrp___protobuf___TextEditingAttributesMessage_pb2___TextEditingAttributes: ...

    def __init__(self,
        *,
        state : typing___Optional[int] = None,
        attributes : typing___Optional[pyatv___mrp___protobuf___TextEditingAttributesMessage_pb2___TextEditingAttributes] = None,
        encryptedTextCyphertext : typing___Optional[bytes] = None,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> KeyboardMessage: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"attributes",u"encryptedTextCyphertext",u"state"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"attributes",u"encryptedTextCyphertext",u"state"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"attributes",b"attributes",u"encryptedTextCyphertext",b"encryptedTextCyphertext",u"state",b"state"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"attributes",b"attributes",u"encryptedTextCyphertext",b"encryptedTextCyphertext",u"state",b"state"]) -> None: ...

keyboardMessage = ... # type: google___protobuf___descriptor___FieldDescriptor
