# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.descriptor import (
    Descriptor as google___protobuf___descriptor___Descriptor,
    FieldDescriptor as google___protobuf___descriptor___FieldDescriptor,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


class CryptoPairingMessage(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    pairingData = ... # type: bytes
    status = ... # type: int
    isRetrying = ... # type: bool
    isUsingSystemPairing = ... # type: bool
    state = ... # type: int

    def __init__(self,
        *,
        pairingData : bytes,
        status : int,
        isRetrying : bool,
        isUsingSystemPairing : bool,
        state : int,
        ) -> None: ...
    @classmethod
    def FromString(cls, s: bytes) -> CryptoPairingMessage: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"isRetrying",u"isUsingSystemPairing",u"pairingData",u"state",u"status"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"isRetrying",u"isUsingSystemPairing",u"pairingData",u"state",u"status"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"isRetrying",b"isRetrying",u"isUsingSystemPairing",b"isUsingSystemPairing",u"pairingData",b"pairingData",u"state",b"state",u"status",b"status"]) -> bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"isRetrying",b"isRetrying",u"isUsingSystemPairing",b"isUsingSystemPairing",u"pairingData",b"pairingData",u"state",b"state",u"status",b"status"]) -> None: ...

cryptoPairingMessage = ... # type: google___protobuf___descriptor___FieldDescriptor
